﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 6.7.2016
 * Time: 10.24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace Pshais.Indicators
{

    public class pshaisLRSx2 : IIndicatorScript
    {
        int length1 = 100, length2 = 300;

        double value1, value2;

        #region IIndicatorScript implementation
        public void Init()
        {

        }
        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument; // Its pre-created for speed based on the "TicksBack" property field

                // Lets do the calculation
                int position = instrument.Close.Count() - (1 + context.Offset); // minus 1 because of counting from 0 + account for the backtest
                var price = instrument.Close[position];

                int beg1, len1;
                var output1 = new double[position];
                var ret1 = Core.LinearReg(1, position, instrument.Close, length1, out beg1, out len1, output1);

                int beg2, len2;
                var output2 = new double[position];
                var ret2 = Core.LinearReg(1, position, instrument.Close, length2, out beg2, out len2, output2);

                if (ret1 == Core.RetCode.Success && ret2 == Core.RetCode.Success)
                {
                    value1 = output1[len1 - 1];
                    value2 = output2[len2 - 1];

                    if (value1 > value2)
                        return IndicatorResult.BuyLong;
                    if (value1 < value2)
                        return IndicatorResult.SellShort;
                }
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }

        public List<ScriptParameter> GetParameters()
        {
            var rtn = new List<ScriptParameter>
            {
                new ScriptParameter("Fast Length", ScriptParameterType.Integer, length1.ToString()),
                new ScriptParameter("Slow Length", ScriptParameterType.Integer, length2.ToString())
            };


            return rtn;
        }
        public void SetParameters(Dictionary<string, object> parameters)
        {
            // Parse fast MA lengths
            length1 = ParseInt(parameters["Fast Length"], length1);
            length2 = ParseInt(parameters["Slow Length"], length2);
        }
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "LR 1", HexLineColor = "#3377AA" },
                new ChartRecord() { ChartIndex = 0, Name = "LR 2", HexLineColor = "#AA7733" }
            };

            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            var chartData = new List<decimal>
            {
                (decimal)value1,
                (decimal)value2
            };

            return chartData;
        }
        public string Name
        {
            get
            {
                return "pshai's Linear Regression x2";
            }
        }
        public int TicksBack
        {
            get
            {
                var lb1 = Core.LinearRegLookback(length1);
                var lb2 = Core.LinearRegLookback(length2);
                return Math.Max(lb1, lb2) + 1;
            }
        }
        #endregion

        int[] ParseInts(object values, int[] lastValues)
        {
            var split = values.ToString().Split('/');

            if (split.Length < lastValues.Length)
                return lastValues;

            try
            {
                var newValues = new int[lastValues.Length];

                for (int i = 0; i < lastValues.Length; i++)
                {
                    newValues[i] = ParseInt(split[i], lastValues[i]);
                }

                return newValues;
            }
            catch
            {
                return lastValues;
            }
        }

        double[] ParseDoubles(object values, double[] lastValues)
        {
            var split = values.ToString().Split('/');

            if (split.Length < lastValues.Length)
                return lastValues;

            try
            {
                var newValues = new double[lastValues.Length];

                for (int i = 0; i < lastValues.Length; i++)
                {
                    newValues[i] = ParseDouble(split[i], lastValues[i]);
                }

                return newValues;
            }
            catch
            {
                return lastValues;
            }
        }

        public static Core.MAType ParseMAType(object value)
        {
            Core.MAType val;
            switch (value.ToString().ToLower())
            {
                case "sma":
                    val = Core.MAType.Sma;
                    break;
                case "ema":
                    val = Core.MAType.Ema;
                    break;
                case "dema":
                    val = Core.MAType.Dema;
                    break;
                case "tema":
                    val = Core.MAType.Tema;
                    break;
                case "trima":
                    val = Core.MAType.Trima;
                    break;
                case "t3":
                    val = Core.MAType.T3;
                    break;
                case "mama":
                    val = Core.MAType.Mama;
                    break;
                case "kama":
                    val = Core.MAType.Kama;
                    break;
                case "wma":
                    val = Core.MAType.Wma;
                    break;
                default:
                    val = Core.MAType.Sma;
                    break;
            }

            return val;
        }

        int ParseInt(object value, int lastValue)
        {
            try
            {
                return Convert.ToInt32(value.ToString());
            }
            catch
            {
                return lastValue;
            }
        }

        double ParseDouble(object value, double lastValue)
        {
            var numStyle = System.Globalization.NumberStyles.Any;
            var cult = System.Globalization.CultureInfo.InvariantCulture;

            try
            {
                double output;
                double.TryParse(
                    value.ToString().Replace(',', '.'),
                    numStyle,
                    cult,
                    out output);
                return output;
            }
            catch
            {
                return lastValue;
            }
        }

        string ParseString(object value, string lastValue)
        {
            try
            {
                return Convert.ToString(value);
            }
            catch
            {
                return lastValue;
            }
        }
    }
}